@@ -1,4 +1,4 @@
-from com import View, msgs
+from com import Component, msgs
 from ui.Button import Button
 from ui.ImageButton import ImageButton
 from ui.Label import Label
@@ -6,7 +6,9 @@ from ui.itemview import ThumbableGridView
 from ui.Slider import VSlider
 from ui.dialog import InputDialog
 from ui.dialog import OptionDialog
+from ui.dialog import ListDialog
 from ui.Pixmap import Pixmap
+from ui.Window import Window
 from StationItem import StationItem
 from FMScale import FMScale
 from FMRadio import *
@@ -24,7 +26,16 @@ except:
     pass
 
 
-class FMRadioUI(View):
+_DEVICE_ERRORS = [
+    "",
+    "Error: Cannot activate Bluetooth/FM combo chip",
+    "Error: Cannot establish communication to the FM receiver",
+    "Error: Cannot load FM radio driver module",
+    "Error: Cannot power up FM radio"
+]
+
+
+class FMRadioUI(Component, Window):
     """
     Player for FM radio stations.
     """
@@ -39,22 +50,24 @@ class FMRadioUI(View):
 
         self.__sound_pipe = SoundPipe()
         self.__use_speaker = False
-        self.__was_playing_before_disconnect = False
+        self.__was_playing_before_interruption = False
         self.__have_headphones = False
         
         self.__stations = maemostations.get_stations()
         self.__stations_dirty = False
         
 
-        View.__init__(self)
+        Component.__init__(self)
+        Window.__init__(self, Window.TYPE_TOPLEVEL)
         self.set_title("FM Radio")
         self.set_visible(True)
         self.connect_closed(self.__on_close_window)
+        self.connect_key_pressed(self.__on_key_pressed)
         
         self.__update_app_menu()
         
         # [Add] button
-        self.__btn_add = Button("Add Station")
+        self.__btn_add = Button("Save Station")
         self.__btn_add.connect_clicked(self.__on_btn_add)
         self.add(self.__btn_add)
 
@@ -67,28 +80,29 @@ class FMRadioUI(View):
         # [Select] button
         self.__btn_select = ImageButton(theme.mb_btn_bookmark_1,
                                         theme.mb_btn_bookmark_2)
-        self.__btn_select.connect_clicked(self.__on_toggle_view)
+        self.__btn_select.connect_clicked(self.__on_btn_select) #self.__on_toggle_view)
         self.add(self.__btn_select)
 
         # stations list
-        self.__stations_list = ThumbableGridView()
-        self.__stations_list.set_visible(False)
-        self.add(self.__stations_list)
+        #self.__stations_list = ThumbableGridView()
+        #self.__stations_list.set_visible(False)
+        #self.add(self.__stations_list)
         
         # slider for scrolling
-        self.__slider = VSlider(theme.mb_list_slider)
-        self.__slider.set_visible(False)
-        self.add(self.__slider)
-        self.__stations_list.associate_with_slider(self.__slider)
+        #self.__slider = VSlider(theme.mb_list_slider)
+        #self.__slider.set_visible(False)
+        #self.add(self.__slider)
+        #self.__stations_list.associate_with_slider(self.__slider)
 
         # FM scale
         self.__fmscale = FMScale()
         self.__fmscale.set_range(87500, 108000)
         self.__fmscale.set_stations(*self.__stations)
         self.__fmscale.connect_frequency_changed(self.__on_tune)
+        self.__fmscale.connect_scanning_activated(self.__on_scan)
         self.add(self.__fmscale)
 
-        self.__lbl_mhz = Label("87.5 MHz", theme.font_fmradio_mhz_label,
+        self.__lbl_mhz = Label("87.5", theme.font_fmradio_mhz_label,
                                theme.color_fmradio_mhz_label)
         self.__lbl_mhz.set_alignment(Label.CENTERED)
         self.add(self.__lbl_mhz)
@@ -120,24 +134,25 @@ class FMRadioUI(View):
         screen = self.get_screen()
         
         screen.fill_area(x, y, w, h, theme.color_mb_background)
+        #screen.fill_area((w - 4) / 2, 0, 4, 100,
+        #                 theme.color_fmradio_scale_needle)
 
-        if (self.__stations_list.is_visible()):
-            self.__stations_list.set_geometry(40, 0, w - 80, h)
-            self.__slider.set_geometry(0, 0, 40, h)
+        #if (self.__stations_list.is_visible()):
+        #    self.__stations_list.set_geometry(40, 0, w - 80, h)
+        #    self.__slider.set_geometry(0, 0, 40, h)
         
+        if (self.__stations):
+            self.__btn_select.set_visible(True)
         else:
-            if (self.__stations):
-                self.__btn_select.set_visible(True)
-            else:
-                self.__btn_select.set_visible(False)
+            self.__btn_select.set_visible(False)
 
-            self.__btn_select.set_geometry(w - 80, h - 80, 80, 80)
-            self.__btn_add.set_geometry(80, h - 80, w - 160, 80)
-            self.__btn_station.set_geometry(80, h - 80, w - 160, 80)
+        self.__btn_select.set_geometry(w - 80, h - 80, 80, 80)
+        self.__btn_add.set_geometry(80, h - 80, w - 160, 80)
+        self.__btn_station.set_geometry(80, h - 80, w - 160, 80)
 
-            self.__btn_play.set_geometry(0, h - 80, 80, 80)
-            self.__fmscale.set_geometry(0, 10, w, h - 80 - 60 - 10)
-            self.__lbl_mhz.set_geometry(0, h - 80 - 50, w, 48)
+        self.__btn_play.set_geometry(0, h - 80, 80, 80)
+        self.__fmscale.set_geometry(0, 80, w, h - 80 - 80 - 1)
+        self.__lbl_mhz.set_geometry(0, 20, w, 48)
         
         screen.draw_pixbuf(theme.fmradio_needle, (w - 24) / 2, h - 80 - 60)
 
@@ -194,17 +209,19 @@ class FMRadioUI(View):
         #end if
 
 
+    """
     def __update_stations_list(self):
     
         self.__stations_list.clear_items()
         for f, n in self.__stations:
             item = StationItem(n, f)
-            item.connect_selected(self.__on_select_item)
             self.__stations_list.append_item(item)
         #end for
         self.__stations_list.invalidate()
+    """
 
 
+    """
     def __toggle_view(self):
     
         if (self.__stations_list.is_visible()):
@@ -232,7 +249,7 @@ class FMRadioUI(View):
         self.__prepare_buffer()
         self.render_at(self.__buffer)
         self.fx_slide_vertical(self.__buffer, 0, 0, w, h, slide_direction)
-        
+    """     
 
 
     def __on_close_window(self):
@@ -255,6 +272,17 @@ class FMRadioUI(View):
         gobject.idle_add(f)
 
 
+    def __on_key_pressed(self, key):
+    
+        if (key == "Left"):
+            self.__fmscale.step_down()
+            
+        elif (key == "Right"):
+            self.__fmscale.step_up()
+
+            
+
+
     def __on_menu_info(self):
     
         hildon.hildon_banner_show_information(self.get_gtk_window(), "",
@@ -269,13 +297,7 @@ class FMRadioUI(View):
 
         if (self.__radio):
             self.__sound_pipe.use_speaker(self.__use_speaker)
-            
-
-    def __on_select_item(self, freq):
-    
-        self.__on_tune(freq)
-        gobject.idle_add(self.__toggle_view)
-        
+      
         
     def __on_tune(self, v):
     
@@ -285,38 +307,48 @@ class FMRadioUI(View):
                 
             self.__set_frequency(v)
         #end if
-        self.__lbl_mhz.set_text("%0.1f MHz" % (v / 1000.0))
+        self.__lbl_mhz.set_text("%0.1f" % (v / 1000.0))
         self.__update_buttons()
 
         if (self.__fmscale.get_value() != v):
             self.__fmscale.set_value(v)
 
         # hilight list item
-        self.__stations_list.set_hilight(-1)
-        cnt = 0
-        for item in self.__stations_list.get_items():
-            if (item.get_frequency() == v):
-                self.__stations_list.set_hilight(cnt)
-                break
-            cnt += 1
+        #self.__stations_list.set_hilight(-1)
+        #cnt = 0
+        #for item in self.__stations_list.get_items():
+        #    if (item.get_frequency() == v):
+        #        self.__stations_list.set_hilight(cnt)
+        #        break
+        #    cnt += 1
         #end for
-        self.__stations_list.invalidate()
-        self.__stations_list.render()
+        #self.__stations_list.invalidate()
+        #self.__stations_list.render()
 
         self.set_title(self.__get_current_station_name() or
                        "%0.1f MHz" % (v / 1000.0))
 
 
-    def __on_scan(self, freq):
+    def __on_scan(self, direction):
     
-        low, hi = self.__radio.get_frequency_range()
-        #gobject.timeout_add(5, lambda :False)
-        gtk.main_iteration(False)
+        def f(freq):
+            self.__fmscale.set_value(freq)
+            gtk.main_iteration(False)
+    
+        if (self.__radio):
+            if (direction < 0):
+                self.__radio.scan_previous(f)
+            else:
+                self.__radio.scan_next(f)
+            #self.__fmscale.set_value(self.__radio.get_frequency())
+        #end if
 
 
+    """
     def __on_toggle_view(self):
     
         self.__toggle_view()
+    """
 
 
     def __on_btn_play(self):
@@ -333,18 +365,6 @@ class FMRadioUI(View):
         #end if
 
 
-    def __on_btn_previous(self):
-
-        if (self.__radio):
-            self.__radio.scan_previous(self.__on_scan)
-
-        
-    def __on_btn_next(self):
-    
-        if (self.__radio):
-            self.__radio.scan_next(self.__on_scan)
-
-
     def __on_btn_add(self):
         
         freq = self.__fmscale.get_value()
@@ -355,7 +375,6 @@ class FMRadioUI(View):
         if (ret == 0):
             name = dlg.get_values()[0] or "%0.1f MHz" % (freq / 1000.0)
             self.__stations.append((freq, name))
-            self.__stations.sort(lambda a,b:cmp(a[0], b[0]))
             self.__fmscale.set_stations(*self.__stations)
             self.__update_stations_list()
             self.__update_buttons()
@@ -379,6 +398,24 @@ class FMRadioUI(View):
                 self.__on_delete_station()
 
 
+    def __on_btn_select(self):
+    
+        self.__stations.sort(lambda a,b:cmp(a[1], b[1]))
+
+        dlg = ListDialog("Favorites")
+        for freq, name in self.__stations:
+            item = StationItem(name, freq)
+            dlg.add_item(item)
+        #end for
+        
+        ret = dlg.run()
+        if (ret == 0):
+            item = dlg.get_choice()
+            freq = item.get_frequency()
+            self.__on_tune(freq)
+        #end if
+
+
     def __on_delete_station(self):
     
         dlg = OptionDialog("Delete Station?")
@@ -454,7 +491,18 @@ class FMRadioUI(View):
             return False
         else:
             return True
-            
+
+
+    def __show_error(self, err):
+    
+        msg = _DEVICE_ERRORS[err]
+    
+        try:
+            hildon.hildon_banner_show_information(self.get_gtk_window(),
+                                                  "", msg)
+        except:
+            pass
+
 
     def __radio_on(self):
 
@@ -463,20 +511,23 @@ class FMRadioUI(View):
         if (self.__device == "RX-51"):
             ret, dev = platforms.request_fmradio()
         else:
-            ret = 0
+            ret = 1
 
         if (ret == 0):
             try:
                 self.__radio = FMRadio(self.__device)
             except:
                 return
+        else:
+            self.__show_error(ret)
+            return
         #end if
         
         if (self.__device == "RX-51"):
             if (self.__keep_alive_handler):
                 gobject.source_remove(self.__keep_alive_handler)
             
-            self.__keep_alive_handler = gobject.timeout_add(60000,
+            self.__keep_alive_handler = gobject.timeout_add(15000,
                                                     self.__keep_radio_alive)
         #end if
 
@@ -530,6 +581,32 @@ class FMRadioUI(View):
         self.__progress.set_message(f.name)
 
 
+    def __begin_interruption(self):
+        """
+        Begins an interruption and switches off the radio, if on.
+        """
+        
+        if (self.__radio):
+            self.__was_playing_before_interruption = True
+            try:
+                hildon.hildon_banner_show_information(self.get_gtk_window(), "",
+                        "FM Radio Interrupted")
+            except:
+                pass
+        else:
+            self.__was_playing_before_interruption = False
+    
+        self.__radio_off()
+
+
+    def __end_interruption(self):
+        """
+        Resumes after an interruption end.
+        """
+
+        if (self.__was_playing_before_interruption):
+            self.__radio_on()
+ 
 
     def handle_CORE_EV_FOLDER_INVALIDATED(self, f):
     
@@ -539,20 +616,13 @@ class FMRadioUI(View):
     def handle_SYSTEM_EV_HEADPHONES_INSERTED(self):
     
         self.__have_headphones = True
-        if (self.__was_playing_before_disconnect):
-            self.__radio_on()
+        self.__end_interruption()
         
         
     def handle_SYSTEM_EV_HEADPHONES_REMOVED(self):
 
         self.__have_headphones = False
-        if (self.__radio):
-            self.__was_playing_before_disconnect = True
-        else:
-            self.__was_playing_before_disconnect = False
-    
-        # no point in playing radio without headphones
-        self.__radio_off()
+        self.__begin_interruption()
 
 
     def handle_SYSTEM_EV_SOUND_VOLUME_CHANGED(self, v):
@@ -561,3 +631,18 @@ class FMRadioUI(View):
         # have to set that manually
         self.__sound_pipe.set_speaker_volume(v)
 
+
+    def handle_SYSTEM_EV_PHONE_RINGING(self):
+    
+        self.__begin_interruption()
+
+
+    def handle_SYSTEM_EV_PHONE_DIALING(self):
+
+        self.__begin_interruption()
+
+
+    def handle_SYSTEM_EV_PHONE_HANGUP(self):
+
+        self.__end_interruption()
+
