@@ -13,10 +13,14 @@ class Window(Widget):
     Base class for windows.
     """
 
+    TYPE_TOPLEVEL = 0
+    TYPE_DIALOG = 1
+
+
     EVENT_CLOSED = "event-closed"
     
 
-    def __init__(self, is_toplevel, hide_cursor = False):
+    def __init__(self, win_type):
         """
         Creates a new window.
         
@@ -24,12 +28,13 @@ class Window(Widget):
         @param hide_cursor: whether to hide the mouse pointer cursor
         """
     
+        self.__type = win_type
         self.__size = (0, 0)
         self.__fixed = None
     
         Widget.__init__(self)
     
-        if (is_toplevel):
+        if (self.__type == self.TYPE_TOPLEVEL):
             if (platforms.PLATFORM in (platforms.MAEMO5, platforms.MER)):
                 import hildon
                 self.__window = hildon.StackableWindow()
@@ -43,8 +48,8 @@ class Window(Widget):
                 self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                 self.__window.set_default_size(800, 480)
             
-        else:
-            self.__window = gtk.Window(gtk.WINDOW_POPUP)
+        elif (self.__type == self.TYPE_DIALOG):
+            self.__window = gtk.Dialog()
 
         self.__window.connect("configure-event", self.__on_configure)
         self.__window.connect("expose-event", self.__on_expose)
@@ -69,6 +74,8 @@ class Window(Widget):
         self.__window.set_app_paintable(True)
         self.__window.realize()
 
+        if (platforms.PLATFORM == platforms.MAEMO5):
+            self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
 
         """
         if (platforms.PLATFORM in (platforms.MAEMO5, platforms.MER)):
@@ -81,9 +88,15 @@ class Window(Widget):
 
         self.__fixed = gtk.Fixed()
         self.__fixed.show()
-        self.__window.add(self.__fixed)
+
+        if (self.__type == self.TYPE_TOPLEVEL):
+            self.__window.add(self.__fixed)
+        else:
+            self.__window.vbox.add(self.__fixed)
+        
         self.__screen = None
 
+        """
         # hide mouse cursor
         if (hide_cursor):
             pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 8, 8)
@@ -92,6 +105,7 @@ class Window(Widget):
             self.__window.window.set_cursor(csr)
             del pbuf
         #end if
+        """
                 
         self.set_window(self)
 
@@ -169,7 +183,7 @@ class Window(Widget):
         else:
             key = gtk.gdk.keyval_name(keyval)
 
-        self.send_event(self.EVENT_KEY_PRESS, key)        
+        self.send_event(self.EVENT_KEY_PRESS, key)
         
         # kill queued events
         if (key in ["Up", "Down", "Left", "Right"]):
